/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JavaxEjbMessageDrivenVisitor.java 2833 2008-04-11 08:09:39Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.deployment.annotations.analyzer.visitors.clazz;

import static org.ow2.easybeans.deployment.annotations.ClassType.MDB;

import org.ow2.easybeans.deployment.annotations.analyzer.AnnotationType;
import org.ow2.easybeans.deployment.annotations.impl.JActivationConfigProperty;
import org.ow2.easybeans.deployment.annotations.impl.JMessageDriven;
import org.ow2.easybeans.deployment.metadata.ejbjar.EjbJarClassMetadata;
import org.ow2.util.scan.api.DefaultAnnotationVisitor;
import org.ow2.util.scan.api.IAnnotationVisitor;

/**
 * This class manages the handling of &#64;{@link javax.ejb.MessageDriven}
 * annotation.
 * @author Florent Benoit
 */
public class JavaxEjbMessageDrivenVisitor extends AbsCommonEjbVisitor<JMessageDriven> implements AnnotationType {

    /**
     * Type of annotation.
     */
    public static final String TYPE = "Ljavax/ejb/MessageDriven;";

    /**
     * messageListenerInterface attribute of the annotation.
     */
    private static final String MESSAGE_LISTENER_INTERFACE = "messageListenerInterface";

    /**
     * activationConfig attribute of the annotation.
     */
    private static final String ACTIVATION_CONFIG = "activationConfig";

    /**
     * Message listener Interface.
     */
    private String messageListenerInterface = null;

    /**
     * Constructor.
     * @param classAnnotationMetadata linked to a class metadata
     */
    public JavaxEjbMessageDrivenVisitor(final EjbJarClassMetadata classAnnotationMetadata) {
        super(classAnnotationMetadata);
    }

    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link Type}.
     */
    @Override
    public void visit(final String name, final Object value) {
        super.visit(name, value);
        if (name.equals(MESSAGE_LISTENER_INTERFACE)) {

            // FIXME how use Type ? asm
//            this.messageListenerInterface = ((Type) value).getInternalName();
        }
    }

    /**
     * Visits an enumeration value of the annotation.
     * @param name the value name.
     * @param desc the class descriptor of the enumeration class.
     * @param value the actual enumeration value.
     */
    @Override
    public void visitEnum(final String name, final String desc, final String value) {

    }

    /**
     * Visits a nested annotation value of the annotation.
     * @param name the value name.
     * @param desc the class descriptor of the nested annotation class.
     * @return a non null visitor to visit the actual nested annotation value.
     *         <i>The nested annotation value must be fully visited before
     *         calling other methods on this annotation visitor</i>.
     */
    @Override
    public IAnnotationVisitor visitAnnotation(final String name, final String desc) {
        return this;
    }

    /**
     * Visits an array value of the annotation.
     * @param name the value name.
     * @return a non null visitor to visit the actual array value elements. The
     *         'name' parameters passed to the methods of this visitor are
     *         ignored. <i>All the array values must be visited before calling
     *         other methods on this annotation visitor</i>.
     */
    @Override
    public IAnnotationVisitor visitArray(final String name) {
        if (name.equals(ACTIVATION_CONFIG)) {
            return new ActivationConfigPropertyVisitor();
        }
        return this;
    }

    /**
     * Visits the end of the annotation. Creates the object and store it
     */
    @Override
    public void visitEnd() {
        super.visitEnd();

        // message listener interface
        getJCommonBean().setMessageListenerInterface(this.messageListenerInterface);

        // Set type
        getAnnotationMetadata().setClassType(MDB);

    }

    /**
     * @return type of the annotation (its description)
     */
    public String getType() {
        return TYPE;
    }

    /**
     * Classes manages the parsing of activationConfig[] array of &#64;{@link javax.ejb.MessageDriven}
     * annotation.
     * @author Florent Benoit
     */
    class ActivationConfigPropertyVisitor extends DefaultAnnotationVisitor {

        /**
         * Attribute for property name.
         */
        private static final String PROPERTY_NAME = "propertyName";

        /**
         * Attribute for property value.
         */
        private static final String PROPERTY_VALUE = "propertyValue";

        /**
         * Property name.
         */
        private String propertyName = null;

        /**
         * Property value.
         */
        private String propertyValue = null;

        /**
         * Visits a primitive value of the annotation.
         * @param name the value name.
         * @param value the actual value, whose type must be {@link Byte},
         *        {@link Boolean}, {@link Character}, {@link Short},
         *        {@link Integer}, {@link Long}, {@link Float},
         *        {@link Double}, {@link String} or {@link Type}.
         */
        @Override
        public void visit(final String name, final Object value) {
            if (name.equals(PROPERTY_NAME)) {
                this.propertyName = (String) value;
            } else if (name.equals(PROPERTY_VALUE)) {
                this.propertyValue = (String) value;
            }
        }

        /**
         * Visits the end of the annotation. Creates the object and store it
         */
        @Override
        public void visitEnd() {
            // Add an activation property
            JActivationConfigProperty jActivationConfigProperty = new JActivationConfigProperty(this.propertyName,
                    this.propertyValue);
            getJCommonBean().addActivationConfigProperty(jActivationConfigProperty);

        }

    }

    /**
     * @return the object representing common bean.
     */
    @Override
    public JMessageDriven getJCommonBean() {
        JMessageDriven jMessageDriven = getAnnotationMetadata().getJMessageDriven();
        if (jMessageDriven == null) {
            jMessageDriven = new JMessageDriven();
            getAnnotationMetadata().setJMessageDriven(jMessageDriven);
        }
        return jMessageDriven;
    }

}
