/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AbsEnvironment.java 1970 2007-10-16 11:49:25Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.deployment.xml.struct.common;

import java.util.ArrayList;
import java.util.List;

/**
 * This class defines an implementation for an environment element.
 * @author Florent Benoit
 */
public class AbsEnvironment {

    /**
     * List of &lt;env-entry&gt; elements.
     */
    private List<EnvEntry> envEntryList = null;

    /**
     * List of &lt;ejb-ref&gt; elements.
     */
    private List<EJBRef> ejbRefList = null;

    /**
     * List of &lt;ejb-local-ref&gt; elements.
     */
    private List<EJBLocalRef> ejbLocalRefList = null;

    /**
     * List of &lt;resource-ref&gt; elements.
     */
    private List<ResourceRef> resourceRefList = null;

    /**
     * List of &lt;resource-env-ref&gt; elements.
     */
    private List<ResourceEnvRef> resourceEnvRefList = null;

    /**
     * List of &lt;message-destination-ref&gt; elements.
     */
    private List<MessageDestinationRef> messageDestinationRefList = null;

    /**
     * List of &lt;around-invoke&gt; elements.
     */
    private List<AroundInvoke> aroundInvokeList = null;

    /**
     * List of &lt;post-activate&gt; elements.
     */
    private List<LifeCycleCallback> postActivateList = null;

    /**
     * List of &lt;pre-passivate&gt; elements.
     */
    private List<LifeCycleCallback> prePassivateList = null;

    /**
     * List of &lt;post-construct&gt; elements.
     */
    private List<LifeCycleCallback> postConstructList = null;

    /**
     * List of &lt;pre-destroy&gt; elements.
     */
    private List<LifeCycleCallback> preDestroyList = null;


    /**
     * Constructor : build a new object for environment.
     */
    public AbsEnvironment() {
        this.envEntryList = new ArrayList<EnvEntry>();
        this.ejbRefList = new ArrayList<EJBRef>();
        this.ejbLocalRefList = new ArrayList<EJBLocalRef>();
        this.resourceRefList = new ArrayList<ResourceRef>();
        this.resourceEnvRefList = new ArrayList<ResourceEnvRef>();
        this.messageDestinationRefList = new ArrayList<MessageDestinationRef>();
        this.aroundInvokeList = new ArrayList<AroundInvoke>();
        this.postActivateList = new ArrayList<LifeCycleCallback>();
        this.postConstructList = new ArrayList<LifeCycleCallback>();
        this.preDestroyList = new ArrayList<LifeCycleCallback>();
        this.prePassivateList = new ArrayList<LifeCycleCallback>();
    }

    /**
     * Add a new env-entry element to this object.
     * @param envEntry the ejb-ref object
     */
    public void addEnvEntry(final EnvEntry envEntry) {
        envEntryList.add(envEntry);
    }

    /**
     * @return the list of all env-entry elements
     */
    public List<EnvEntry> getEnvEntryList() {
        return envEntryList;
    }


    /**
     * Gets the list of &lt;around-invoke&gt; elements.
     * @return list of &lt;around-invoke&gt; elements.
     */
    public List<AroundInvoke> getAroundInvokeList() {
        return aroundInvokeList;
    }

    /**
     * Adds a new &lt;around-invoke&gt; element to assembly descriptor.
     * @param aroundInvoke the &lt;around-invoke&gt; element
     */
    public void addAroundInvoke(final AroundInvoke aroundInvoke) {
        aroundInvokeList.add(aroundInvoke);
    }

    /**
     * Gets the list of &lt;post-construct&gt; elements.
     * @return list of &lt;post-construct&gt; elements.
     */
    public List<LifeCycleCallback> getPostConstructCallbackList() {
        return postConstructList;
    }

    /**
     * Adds a new &lt;post-construct&gt; element to assembly descriptor.
     * @param postConstructCallback the &lt;post-construct&gt; element
     */
    public void addPostConstructCallback(final LifeCycleCallback postConstructCallback) {
        postConstructList.add(postConstructCallback);
    }

    /**
     * Gets the list of &lt;post-activate&gt; elements.
     * @return list of &lt;post-activate&gt; elements.
     */
    public List<LifeCycleCallback> getPostActivateCallbackList() {
        return postActivateList;
    }

    /**
     * Adds a new &lt;post-activate&gt; element to assembly descriptor.
     * @param postActivateCallback the &lt;post-activate&gt; element
     */
    public void addPostActivateCallback(final LifeCycleCallback postActivateCallback) {
        postActivateList.add(postActivateCallback);
    }

    /**
     * Gets the list of &lt;pre-destroy&gt; elements.
     * @return list of &lt;pre-destroy&gt; elements.
     */
    public List<LifeCycleCallback> getPreDestroyCallbackList() {
        return preDestroyList;
    }

    /**
     * Adds a new &lt;pre-destroy&gt; element to assembly descriptor.
     * @param preDestroyCallback the &lt;pre-destroy&gt; element
     */
    public void addPreDestroyCallback(final LifeCycleCallback preDestroyCallback) {
        preDestroyList.add(preDestroyCallback);
    }

    /**
     * Gets the list of &lt;pre-passivate&gt; elements.
     * @return list of &lt;pre-passivate&gt; elements.
     */
    public List<LifeCycleCallback> getPrePassivateCallbackList() {
        return prePassivateList;
    }

    /**
     * Adds a new &lt;pre-passivate&gt; element to assembly descriptor.
     * @param prePassivateCallback the &lt;pre-passivate&gt; element
     */
    public void addPrePassivateCallback(final LifeCycleCallback prePassivateCallback) {
        prePassivateList.add(prePassivateCallback);
    }

    /**
     * Gets the list of &lt;ejb-ref&gt; elements.
     * @return list of &lt;ejb-ref&gt; elements.
     */
    public List<EJBRef> getEJBRefList() {
        return ejbRefList;
    }

    /**
     * Adds a new &lt;ejb-ref&gt; element.
     * @param ejbRef the &lt;ejb-ref&gt; element
     */
    public void addEJBRef(final EJBRef ejbRef) {
        ejbRefList.add(ejbRef);
    }

    /**
     * Gets the list of &lt;resource-ref&gt; elements.
     * @return list of &lt;resource-ref&gt; elements.
     */
    public List<ResourceRef> getResourceRefList() {
        return resourceRefList;
    }

    /**
     * Adds a new &lt;resource-ref&gt; element.
     * @param resourceRef the &lt;resource-ref&gt; element
     */
    public void addResourceRef(final ResourceRef resourceRef) {
        resourceRefList.add(resourceRef);
    }

    /**
     * Gets the list of &lt;message-destination-ref&gt; elements.
     * @return list of &lt;message-destination-ref&gt; elements.
     */
    public List<MessageDestinationRef> getMessageDestinationRefList() {
        return messageDestinationRefList;
    }

    /**
     * Adds a new &lt;message-destination-ref&gt; element.
     * @param messageDestinationRef the &lt;message-destination-ref&gt; element
     */
    public void addMessageDestinationRef(final MessageDestinationRef messageDestinationRef) {
        messageDestinationRefList.add(messageDestinationRef);
    }


    /**
     * Gets the list of &lt;resource-env-ref&gt; elements.
     * @return list of &lt;resource-env-ref&gt; elements.
     */
    public List<ResourceEnvRef> getResourceEnvRefList() {
        return resourceEnvRefList;
    }

    /**
     * Adds a new &lt;resource-ref&gt; element.
     * @param resourceEnvRef the &lt;resource-ref&gt; element
     */
    public void addResourceEnvRef(final ResourceEnvRef resourceEnvRef) {
        resourceEnvRefList.add(resourceEnvRef);
    }

    /**
     * Gets the list of &lt;ejb-local-ref&gt; elements.
     * @return list of &lt;ejb-local-ref&gt; elements.
     */
    public List<EJBLocalRef> getEJBLocalRefList() {
        return ejbLocalRefList;
    }

    /**
     * Adds a new &lt;ejb-local-ref&gt; element.
     * @param ejLocalRef the &lt;ejb-local-ref&gt; element
     */
    public void addEJBLocalRef(final EJBLocalRef ejLocalRef) {
        ejbLocalRefList.add(ejLocalRef);
    }
}
